# This is the example from Jelena Ilic presentation Modelling, Tidyverse Way
# https://github.com/satRdays/newcastle2019-slides/blob/master/03-ilic-modelling-tidyverse.pdf
# The data is from 
# https://archive.ics.uci.edu/ml/datasets/auto+mpg
# https://www.kaggle.com/uciml/autompg-dataset

# TidyModels
# https://www.tidymodels.org

# Tidy Modeling with R
# https://www.tmwr.org


library(tidyverse)
library(tidymodels)
library(janitor)
library(broom)

# Load
autompg <- read_csv("auto-mpg.csv") 
head(autompg)

autompg <- autompg %>% clean_names()
head(autompg) 

# keep only the model from from the car_name

autompg <- autompg %>% separate(car_name, c("make"), sep = "\\s") %>%
    mutate(horsepower = as.numeric(horsepower), origin = factor(origin))
head(autompg) 

#--------------------------------- Splitting ----------------------------------#
# Package: rsample

set.seed(42)
auto_split <- initial_split(autompg, prop = 0.8, strata = "mpg")
auto_train <- training(auto_split)
auto_test <- testing(auto_split)

#--------------------------------- Preprocessing ------------------------------#
# Package recipes 

# 
# The recipes package rigidly follows a cooking analogy for applying preprocessing 
# steps, which is totally analogous to how we fit a model. The three stages are:
# 
# 1.	recipe: Define a specification for preprocessing steps.  none of the 
# actual data.
# 
# 2.	prep:  For a recipe with at least one preprocessing operation, estimate the 
# required parameters from a training set that can be later applied to other data sets.
# 
# 3.	bake: Apply the recipe 
rec_auto <- recipe(mpg ~ ., data = auto_train) %>%
  step_other(make, threshold = 0.05) %>%  # move infrequently occurring values into the "other" category.
  step_dummy(origin, make) %>%            # deal with categories
  # step_center(all_predictors()) %>%     # uncomment for other models than lm()
  # step_scale(all_predictors()) %>%
  step_knnimpute(all_predictors()) %>%    # impute missing data using nearest neighbors
  prep(auto_train)                        # prepare the recipe

# bake 
# Apply the recipe on the training set
auto_train <- juice(rec_auto) 

# Apply the recipe on the test set
auto_test <- bake(rec_auto, auto_test)

# ------------------Create a model , choose the enfine and fit------------------#
# Package parsnip

# the algorithm and the computational machine are detached
# the data do not enter the play until the fit step

show_engines("linear_reg")

model <- linear_reg() %>%
  set_engine("lm") %>%
  fit(mpg ~ ., data = auto_train)

# model <- linear_reg(penalty = 0.01) %>%
#  set_engine("glmnet") %>%
#  fit(mpg ~ ., data = auto_train)

# show_engines("boost_tree")

# model <- boost_tree() %>%
#  set_engine("xgboost") %>%
#  set_mode("regression") %>%
#  fit(mpg ~ ., data = auto_train)

# show_engines("rand_forest")

# model <- rand_forest() %>%
#  set_engine("ranger") %>% 
#  set_mode("regression") %>% 
#  fit(mpg ~ ., data = auto_train)
  
tidy(model)

# -------------------------------------- Predict-------------------------------#
# parsnip  model ALWAYS returns the results as a data frame 


predictions <- bind_cols(
  select(auto_test, mpg),
  predict(model, auto_test)
)

head(predictions)

# -------------------Score Get the metrics ------------------------------------#
# And here comes yardstick
# When "truth" is a factor: accuracy and the Kappa
# When "truth" is numeric: rmse, rsq and mae.

predictions %>% ggplot(aes(x = mpg, y = .pred)) +
  geom_point() +
  geom_smooth(method = lm) +
  ggtitle("Predicted values vs True values for the Test data")

metrics(predictions, truth = mpg, estimate = .pred)
